function exportModel(model,fileName)
% exportModel
%   Exports a constraint-based model to a SBML file
%
%   model         a model structure
%   fileName      the filename  to export the model to
%
%   The resulting SBML file is in the format defined in the yeast consensus
%   model. No checks at all are made to ensure the correctness of the
%   model.
%   
%   NOTE: This function is currently not able to deal with gene complexes
%         and saves them only as 'or' relationships. Use
%         exportToExcelFormat if you have gene complexes in your model.
%
%   Usage: exportModel(model,fileName)
%
%   Rasmus Agren, 2013-02-06
%

%Check if the "unconstrained" field is still present. This shows if
%exchange metabolites have been removed
if ~isfield(model,'unconstrained')
   fprintf('WARNING: There is no unconstrained field in the model structure. This means that no metabolites are considered exchange metabolites\n');
   model.unconstrained=zeros(numel(model.mets),1);
end

%Check if genes have associated compartments. This is only needed for the
%new format and has no real impact on anything. 
if ~isfield(model,'geneComps') && isfield(model,'genes')
	fprintf('WARNING: There are no compartments specified for the genes. All genes will be assigned to a new compartment named "Unspecified" and abbreviated "u".\n');
    model.comps=[model.comps;{'u'}];
    model.compNames=[model.compNames;{'Unspecified'}];

    if isfield(model,'compOutside')
    	model.compOutside=[model.compOutside;{''}];
    end
    if isfield(model,'compMiriams')
    	model.compMiriams=[model.compMiriams;{[]}];
    end
    geneComps=cell(numel(model.genes),1);
    geneComps(:,:)=cellstr('u');
    model.geneComps=geneComps;
end

%Generate temporary filename
tempFile=tempname;

%Open a stream
fid = fopen(tempFile,'w');

%Writes the intro
intro=['<?xml version="1.0" encoding="UTF-8" ?>'...
'<sbml xmlns="http://www.sbml.org/sbml/level2/version3" level="2" version="3">'...
'<model metaid="metaid_' model.id '" id="' model.id '" name="' model.description '">'...
'<notes>'...
'<body xmlns="http://www.w3.org/1999/xhtml">'...
'<p>This file was generated using the exportModel function in RAVEN Toolbox.</p>'...
'</body>'...
'</notes>'...
'<annotation>'...
'<rdf:RDF xmlns:rdf="http://www.w3.org/1999/02/22-rdf-syntax-ns#" xmlns:dc="http://purl.org/dc/elements/1.1/" xmlns:dcterms="http://purl.org/dc/terms/" xmlns:vCard="http://www.w3.org/2001/vcard-rdf/3.0#" xmlns:bqbiol="http://biomodels.net/biology-qualifiers/" xmlns:bqmodel="http://biomodels.net/model-qualifiers/">'...
'<rdf:Description rdf:about="#metaid_' model.id '">'...
'<dc:creator rdf:parseType="Resource">'...
'<rdf:Bag>'...
'<rdf:li rdf:parseType="Resource">'...
'<vCard:N rdf:parseType="Resource">'...
'<vCard:Family>Agren</vCard:Family>' ...
'<vCard:Given>Rasmus</vCard:Given>' ...
'</vCard:N>'...
'<vCard:EMAIL>rasmus.agren@chalmers.se</vCard:EMAIL>'... 
'<vCard:ORG>'...
'<vCard:Orgname>Chalmers University of Technology, Gothenburg</vCard:Orgname>'... 
'</vCard:ORG>'...
'</rdf:li>'...
'</rdf:Bag>'...
'</dc:creator>'...
'<dcterms:created rdf:parseType="Resource">'...
'<dcterms:W3CDTF>' datestr(now,'yyyy-mm-ddTHH:MM:SSZ') '</dcterms:W3CDTF>'... 
'</dcterms:created>'...
'<dcterms:modified rdf:parseType="Resource">'...
'<dcterms:W3CDTF>' datestr(now,'yyyy-mm-ddTHH:MM:SSZ') '</dcterms:W3CDTF>'... 
'</dcterms:modified>'...
'</rdf:Description>'...
'</rdf:RDF>'...
'</annotation>'...
'<listOfUnitDefinitions>'...
'<unitDefinition id="mmol_per_gDW_per_hr">'...
'<listOfUnits>'...
'<unit kind="mole" scale="-3"/>'...
'<unit kind="second" multiplier="0.00027778" exponent="-1"/>'...
'</listOfUnits>'...
'</unitDefinition>'...
'</listOfUnitDefinitions>'...
'<listOfCompartments>'];

%Write intro
fprintf(fid,intro);

%Write compartments
for i=1:numel(model.comps)
    %Check if it's outside anything
    if isfield(model, 'compOutside')
        if ~isempty(model.compOutside{i})
            append=[' outside="C_' model.compOutside{i} '" spatialDimensions="3"'];
        else
            append=' spatialDimensions="3"';
        end
    else
        append=' spatialDimensions="3"';
    end
    
    fprintf(fid,['<compartment metaid="metaid_C_' model.comps{i} '" id="C_' model.comps{i} '" name="' model.compNames{i} '"' append ' sboTerm="SBO:0000290">']);

    %Print associated Miriam strings
    if isfield(model,'compMiriams')
        miriamString=getMiriam(model.compMiriams{i});
    else
        miriamString=[];
    end

    if ~isempty(miriamString)
        compinfo=['<annotation><rdf:RDF xmlns:rdf="http://www.w3.org/1999/02/22-rdf-syntax-ns#" xmlns:dc="http://purl.org/dc/elements/1.1/" xmlns:dcterms='...
                '"http://purl.org/dc/terms/" xmlns:vCard="http://www.w3.org/2001/vcard-rdf/3.0#" xmlns:bqbiol="http://biomodels.net/biology-qualifiers/" '...
                'xmlns:bqmodel="http://biomodels.net/model-qualifiers/"><rdf:Description rdf:about="#metaid_C_' model.comps{i} '">'...
                '<bqbiol:is><rdf:Bag>' miriamString '</rdf:Bag></bqbiol:is>'...
                '</rdf:Description></rdf:RDF></annotation></compartment>'];
    else
        compinfo='</compartment>\n';
    end
    fprintf(fid,compinfo);
end

intro='</listOfCompartments><listOfSpecies>';
fprintf(fid,intro);

%Begin writing species
for i=1:numel(model.mets)
    if model.unconstrained(i)
        unbounded='true';
    else
        unbounded='false';
    end
    
    toprint=['<species metaid="metaid_M_' model.mets{i} '" id="M_' model.mets{i} '" name="' model.metNames{i} '" compartment="C_' model.comps{model.metComps(i)} '" boundaryCondition="' unbounded '" sboTerm="SBO:0000299">'];
    
    %Print some stuff if there is a formula for the compound
    if isfield(model,'metFormulas')
        %Only print formula if there is no InChI. The formula is loaded
        %from the InChI-string in importModel. This may have to change!
        
        hasInchi=false;
        if isfield(model,'inchis')
            if ~isempty(model.inchis{i})
                hasInchi=true;
            end
        end
        
        if ~isempty(model.metFormulas{i}) && hasInchi==false
            toprint=[toprint '<notes><body xmlns="http://www.w3.org/1999/xhtml"><p>FORMULA: '  model.metFormulas{i} '</p></body></notes>'];
        end
    end
    
    %Only print annotations for metabolites with some miriam link. This is because I don't know how "unknown" 
    %metabolites should be presented, and it seems unlikely that you will
    %have InChI without a database link. This might be temporary....
    if isfield(model,'metMiriams')
        miriamString=getMiriam(model.metMiriams{i});
    else
        miriamString=[];
    end

    if ~isempty(miriamString)
        toprint=[toprint '<annotation>'];

        %Print InChI if available
        if isfield(model,'inchis')
           if ~isempty(model.inchis{i})
                toprint=[toprint '<in:inchi xmlns:in="http://biomodels.net/inchi" metaid="metaid_M_' model.mets{i} '_inchi">InChI=' model.inchis{i} '</in:inchi>'];
                isInchi=1;
           else
                isInchi=0;
           end
        else
            isInchi=0;
        end
        %Print some more annotation stuff
        toprint=[toprint '<rdf:RDF xmlns:rdf="http://www.w3.org/1999/02/22-rdf-syntax-ns#" xmlns:dc="http://purl.org/dc/elements/1.1/" xmlns:dcterms="http://purl.org/dc/terms/" '...
            'xmlns:vCard="http://www.w3.org/2001/vcard-rdf/3.0#" xmlns:bqbiol="http://biomodels.net/biology-qualifiers/" xmlns:bqmodel="http://biomodels.net/model-qualifiers/">'...
            '<rdf:Description rdf:about="#metaid_M_' model.mets{i} '">'...
            '<bqbiol:is>'...
            '<rdf:Bag>'];
        if isInchi==1
            toprint=[toprint '<rdf:li rdf:resource="#metaid_M_' model.mets{i} '_inchi" />'];
        end

        %Print Miriam and finish up
        toprint=[toprint miriamString '</rdf:Bag></bqbiol:is></rdf:Description></rdf:RDF></annotation>'];
    end
    toprint=[toprint '</species>\n'];
    fprintf(fid,toprint);
end

%Write genes and complexes
%Write genes
if isfield(model,'genes')
    for i=1:numel(model.genes)
       toprint=['<species metaid="metaid_E_' int2str(i) '" id="E_' int2str(i) '" name="' model.genes{i} '" compartment="C_' geneComps{i} '" sboTerm="SBO:0000014">'];

       %Print gene name if present
       if isfield(model,'geneShortNames')
           if ~isempty(model.geneShortNames{i})
                toprint=[toprint '<notes><body xmlns="http://www.w3.org/1999/xhtml"><p>SHORT NAME: '  model.geneShortNames{i} '</p></body></notes>'];
           end
       end

       if isfield(model,'geneMiriams')
            miriamString=getMiriam(model.geneMiriams{i});
       else
            miriamString=[]; 
       end

       if ~isempty(miriamString)
           toprint=[toprint '<annotation><rdf:RDF xmlns:rdf="http://www.w3.org/1999/02/22-rdf-syntax-ns#" xmlns:dc="http://purl.org/dc/elements/1.1/" '...
                'xmlns:dcterms="http://purl.org/dc/terms/" xmlns:vCard="http://www.w3.org/2001/vcard-rdf/3.0#" xmlns:bqbiol="http://biomodels.net/biology-qualifiers/" '...
                'xmlns:bqmodel="http://biomodels.net/model-qualifiers/"><rdf:Description rdf:about="#metaid_E_' int2str(i) '"><bqbiol:is><rdf:Bag>'...
                miriamString '</rdf:Bag></bqbiol:is></rdf:Description></rdf:RDF></annotation>'];
       end

       toprint=[toprint '</species>\n'];
       fprintf(fid,toprint);
    end
end
%Write complexes (NOT DONE YET)

%Finish metbolites
fprintf(fid,'</listOfSpecies>');

%Add reactions
fprintf(fid,'<listOfReactions>');

for i=1:length(model.rxns)
    %Get reversibility
    reversible='false';
    if model.rev(i)==1
        reversible='true';
    end

    %TEMP, MUST FIX SBO-TERM
    fprintf(fid,['<reaction metaid="metaid_R_' model.rxns{i} '" id="R_' model.rxns{i} '" name="' model.rxnNames{i} '" reversible="' reversible '" sboTerm="SBO:0000176">']);

    if isfield(model,'subSystems')
        subsystem=model.subSystems{i};
    else
        subsystem=[];
    end
    
    if ~isempty(subsystem)
        fprintf(fid,['<notes><body xmlns="http://www.w3.org/1999/xhtml"><p>SUBSYSTEM: ' subsystem '</p></body></notes>']);
    end
    
	%Print annotation
    if isfield(model,'rxnMiriams')
    	miriamString=getMiriam(model.rxnMiriams{i});
    else
        miriamString=[];
    end

    %Check to see if there is an associated ec-code
    if isfield(model,'eccodes')
        if ~isempty(model.eccodes{i})
            miriamString=[miriamString  '<rdf:li rdf:resource="urn:miriam:ec-code:' model.eccodes{i} '"/>'];
        end
    end

    if ~isempty(miriamString)
        fprintf(fid,['<annotation><rdf:RDF xmlns:rdf="http://www.w3.org/1999/02/22-rdf-syntax-ns#" '...
        'xmlns:dc="http://purl.org/dc/elements/1.1/" xmlns:dcterms="http://purl.org/dc/terms/" '...
        'xmlns:vCard="http://www.w3.org/2001/vcard-rdf/3.0#" xmlns:bqbiol="http://biomodels.net/biology-qualifiers/" '...
        'xmlns:bqmodel="http://biomodels.net/model-qualifiers/">'...
        '<rdf:Description rdf:about="#metaid_R_' model.rxns{i} '">'...
        '<bqbiol:is>'...
        '<rdf:Bag>'...
        miriamString... 
        '</rdf:Bag>'...
        '</bqbiol:is>'...
        '</rdf:Description>'...
        '</rdf:RDF>'...
        '</annotation>']);
    end
    
    %The reactants have negative values in the stochiometric matrix
    compounds=model.S(:,i);
    reactants=find(compounds<0);
    products=find(compounds>0);
    
    if any(reactants)
        fprintf(fid,'<listOfReactants>');
        for j=1:length(reactants)
            tempmetname=model.mets{reactants(j)};
            fprintf(fid,['<speciesReference species="M_' tempmetname '" stoichiometry="' num2str(-1*compounds(reactants(j))) '"/>']);
        end  

        fprintf(fid,'</listOfReactants>');
    end
    if any(products)
        fprintf(fid,'<listOfProducts>');
        for j=1:length(products)
           tempmetname=model.mets{products(j)};
           fprintf(fid,['<speciesReference species="M_' tempmetname '" stoichiometry="' num2str(compounds(products(j))) '"></speciesReference>']);
        end  
        fprintf(fid,'</listOfProducts>');
    end
    
    genes=find(model.rxnGeneMat(i,:));
    if any(genes)
        fprintf(fid,'<listOfModifiers>');
        for j=1:numel(genes)
            fprintf(fid,['<modifierSpeciesReference species="E_' num2str(genes(j)) '" />']);
        end
        fprintf(fid,'</listOfModifiers>');
    end

    %Print constraints, reversibility, and objective. It's assumed that all 
    %information is present in the model structure. This should be ok, since
    %it should have been set to standard values when imported
    
    %Note that the order of parameters is hard-coded. It's the same thing
    %in importModel

    fprintf(fid,'<kineticLaw><math xmlns="http://www.w3.org/1998/Math/MathML"><ci>FLUX_VALUE</ci></math><listOfParameters>');
    fprintf(fid,['<parameter id="LB_R_' model.rxns{i} '" name="LOWER_BOUND" value="' sprintf('%15.8f',model.lb(i)) '" units="mmol_per_gDW_per_hr"/><parameter id="UB_R_' model.rxns{i} '" name="UPPER_BOUND" value="' sprintf('%15.8f',model.ub(i)) '" units="mmol_per_gDW_per_hr"/><parameter id="OBJ_R_' model.rxns{i} '" name="OBJECTIVE_COEFFICIENT" value="' sprintf('%15.8f',model.c(i)) '" units="dimensionless"/><parameter id="FLUX_VALUE" value="0.00000000" units="mmol_per_gDW_per_hr"/>']);
    fprintf(fid,'</listOfParameters></kineticLaw>');
    fprintf(fid,'</reaction>');
end

%Write outro
outro='</listOfReactions></model></sbml>';
fprintf(fid,outro);

fclose(fid);

%Replace the target file with the temporary file
delete(fileName);
movefile(tempFile,fileName);
end

function miriamString=getMiriam(miriamStruct)
%Returns a string with list elements for a miriam structure ('<rdf:li
%rdf:resource="urn:miriam:obo.go:GO:0005739"/>' for example). This is just
%to speed ut things since this is done many times during the exporting

miriamString='';
if isfield(miriamStruct,'name')
	for i=1:numel(miriamStruct.name)
        miriamString=[miriamString '<rdf:li rdf:resource="urn:miriam:' miriamStruct.name{i} ':' miriamStruct.value{i} '"/>'];
	end
end
end
