function [nC, nP, nS, nN, nR, nO, nH, exitFlag]=parseFormulas(formulas, noPolymers,isInchi)
% parseFormulas
%   Gets the elemental composition from formulas
%
%   formulas      a cell array with formulas
%   noPolymers    assume that all polymers consist of one element.
%                 Corresponds to counting everything between (...)n as
%                 n being equal to one. Only one set of parentheses
%                 is allowed (opt, default false)
%   isInchi       true if the formulas are in the InChI format (opt,
%                 default false)
%
%   nC            Array with the carbon coefficients
%   nP            Array with the phosphorus coefficients
%   nS            Array with the sulfur coefficients
%   nN            Array with the nitrogen coefficients
%   nR            Array with the "general group" coefficents
%   nO            Array with the oxygen coefficients
%   nH            Array with the hydrogen coefficients
%   exitFlag      Array with the exit flags:
%                 1=  Sucessful parsing
%                 0=  No formula found
%                 -1= Could not parse formula
%
%   Returns vectors corresponding to the coefficients for carbon, phosphate, 
%   sulfur, nitrogen, "R-groups", oxygen, and hydrogen. All other compounds 
%   are ignored.
%
%   NOTE: Formulas which contain any two-character compounds (Zn, Fe and so on)
%         might cause problems.
%
%   Usage: [nC, nP, nS, nN, nR, nO, nH, exitFlag]=parseFormulas(formulas,
%           noPolymers,isInchi)
%
%   Rasmus Agren, 2012-12-18
%

if nargin<2
    noPolymers=false;
end
if nargin<3
    isInchi=false;
end

%Only one-character abbreviations work at the moment!
abbrevs=['C';'P';'S';'N';'R';'O';'H'];

coeffs=zeros(numel(formulas),numel(abbrevs));
exitFlag=zeros(numel(formulas),1);

%Loop through all of the formulas. It might be possible to do this with
%vectors, but I think this is more flexible
for i=1:numel(formulas)
    foundError=false;
    
    %If it's empty, go to next loop
    if isempty(formulas{i})
       continue; 
    end
    
    if isInchi==false
        %Check if it's a polymer (has "(" and ")n")
        startPol=strfind(formulas{i},'(');
        endPol=strfind(formulas{i},')n');

        if isempty(startPol) && isempty(endPol)
            splitComp=formulas(i);
        else
            %There must be only one set of parentheses and noPolymers must be
            %true
            if numel(startPol)==1 && numel(endPol)==1 && startPol(1) < endPol(1)...
                    && noPolymers==true
                %Parse the composition
                splitComp{1}=formulas{i}(1:startPol-1);
                splitComp{2}=formulas{i}(startPol+1:endPol-1);
                splitComp{3}=formulas{i}(endPol+2:numel(formulas{i}));
            else
                exitFlag(i)=-1;
                continue;
            end
        end
    else
        %If it's an InChI code. Polymers are not supported in this format,
        %so no checks have to be made
        %The composition is found between the first and the second "/"
        S=regexp(formulas{i},'/','split');
        if numel(S)>2
            splitComp=S(2);
        else
            splitComp={''};
            if numel(S)==2
                %This can happen for simple ions (I think when there are no
                %bindings in the molecule). Normally this doesn't have to be dealt
                %with in any specific way, but one important exception is the
                %proton, which is stored as "p+1" rather than as "H+" which
                %would fit with the rest of the parsing
                if strcmpi(S(2),'p+1')
                    splitComp={'H+'};
                end
            end
        end
    end
    
    %Loop through the parts and sum the coefficients
    for j=1:numel(splitComp)
       %Get the indexes of all non-numeric characters (or ".")
       nonNumeric=regexp(splitComp{j},'[^0-9.]');
       if ~isempty(splitComp{j})
           %Loop through each element
           for k=1:numel(abbrevs)
                index=strfind(splitComp{j},abbrevs(k));
                
                %Only one element should have been found
                if numel(index)==1
                    %Get what should be the coefficient
                    endIndex=find(nonNumeric>index,1);
                    if ~isempty(endIndex)
                        endChar=nonNumeric(endIndex)-1;
                    else
                        endChar=numel(splitComp{j});
                    end
                    
                    %Assume that the coefficient is 1 if followed by a
                    %non-numeric character
                    if endChar==index
                        coeffs(i,k)=coeffs(i,k)+1;
                    else
                        co=str2num(splitComp{j}(index+1:endChar));
                        
                        %This should represent a number. If it doesn't then
                        %something is wrong
                        if ~isempty(co)
                            coeffs(i,k)=coeffs(i,k)+co;
                        else
                            foundError=true;
                            break;
                        end
                    end
                else
                    if numel(index)>1
                        %Can't proceed if it's more than one
                        foundError=true;
                        break;
                    end
                end
           end
       end
       if foundError==true
           break;
       end
    end
    if foundError==true
       exitFlag(i)=-1;
       coeffs(i,:)=0;
    else
        exitFlag(i)=1;
    end
end

nC=coeffs(:,1);
nP=coeffs(:,2);
nS=coeffs(:,3);
nN=coeffs(:,4);
nR=coeffs(:,5);
nO=coeffs(:,6);
nH=coeffs(:,7);
end
