function [newModel removedRxns]=removeBadRxns(model,rxnRules,ignoreMets,isNames,balanceElements,refModel)
% removeBadRxns
%   Iteratively removes reactions which enable production of some
%   metabolite without any uptake
%
%   model                   a model structure. For the intented function,
%                           the model shouldn't allow for any uptake. The
%                           easiest way to achieve this is to import the
%                           model using importModel('filename',false)
%   rxnRules                1: only remove reactions which are unbalanced
%                           2: also remove reactions which couldn't be checked for
%                           mass balancing
%                           3: all reactions can be removed 
%                           (opt, default 1)
%   ignoreMets              either a cell array of metabolite IDs, a logical vector 
%                           with the same number of elements as metabolites in the model,
%                           of a vector of indexes for metabolites to exclude from
%                           this analysis (opt, default [])
%   isNames                 true if the supplied mets represent metabolite names
%                           (as opposed to IDs). This is a way to delete
%                           metabolites in several compartments at once without
%                           knowing the exact IDs. This only works if ignoreMets
%                           is a cell array (opt, default false)
%   balanceElements         a cell array with the elements for which to
%                           balance the reactions. May contain any
%                           combination of 'C','P','S','N','R','O','H'
%                           (where R stands for R group) (opt, default
%                           {'C';'P';'S';'N';'O'}
%   refModel                a reference model which can be used to ensure
%                           that the resulting model is still functional.
%                           The intended use is that the reference model is
%                           a copy of model, but with uptake allowed and
%                           some objectives (such as production of biomass)
%                           constrained to a non-zero flux. Before a
%                           reaction is removed from "model" the function first
%                           checks that the same deletion in "refModel"
%                           doesn't render the problem unfeasible (opt)
%
%   newModel               	a model structure after the problematic
%                           reactions have been deleted
%   removedRxns             a cell array with the reactions that were
%                           removed
%
%   The purpose of this function is to remove reactions which enable
%   production of metabolites even when exchange reactions aren't used.
%   Many models, especially if they are automatically inferred from
%   databases, will have unbalanced reactions which allow for
%   net-production of metabolites without any consumption. A common reason
%   for this is when general compounds have different meaning in different
%   reactions (as DNA has in these two reactions).
%       dATP + dGTP + dCTP + dTTP <=> DNA + 4 PPi
%       0.25 dATP + 0.25 dGTP + 0.25 dCTP + 0.25 dTTP <=> DNA + PPi
%   Reactions that are problematic like this are always elementally
%   unbalanced, but it is not always that you would like to exclude all
%   unbalanced reactions from your model.
%   This function tries to remove as few problematic reactions as possible
%   so that the model cannot produce anything from nothing. This is done by
%   repeatedly calling makeSomething, checking if any of the involved
%   reactions are elementally unbalanced, remove one of them, and then 
%   iterating until no metabolites can be synthesized.
%
%   Usage: [newModel removedRxns]=removeBadRxns(model,rxnRules,...
%       ignoreMets,isNames,refModel)
%
%   Rasmus Agren, 2012-03-20
%

if nargin<2
    rxnRules=1;
end
if nargin<3
    ignoreMets=[];
end
if nargin<4
    isNames=false;
end
if nargin<5
    balanceElements={'C';'P';'S';'N';'O'};
end
if nargin<6
    refModel=[];
end

%Check if the model has open exchange reactions and print a warning in that case
if ~isfield(model,'unconstrained')
    [crap I]=getExchangeRxns(model);
    if any(I)
        if any(model.lb(I)~=0) || any(model.ub(I)~=0)
            fprintf('WARNING: The model contains open exchange reactions. This is not the intended use of this function. Consider importing your model using importModel(filename,false).\n');
        end
    end
end

%Check that the model is feasible
sol=solveLP(model);
if isempty(sol.f)
    throw(MException('','The model is not feasible. Consider removing lower bounds (such as ATP maintenance)')); 
end

%Check that the reference model is feasible
if any(refModel)
    sol=solveLP(refModel);
    if isempty(sol.f)
        throw(MException('','The reference model is not feasible.')); 
    end
end

%Initialize some stuff
removedRxns={};
balanceStructure=getElementalBalance(model);

%Check which elements to balance for
if ~isempty(setdiff(upper(balanceElements),{'C','P','S','N','R','O','H'}))
    throw(MException('','Could not recognize all elements to balance for')); 
end
bal=ismember({'C','P','S','N','R','O','H'},upper(balanceElements));

%Main loop
while 1
    %Make some metabolite using as few reactions as possible
    [solution metabolite]=makeSomething(model,ignoreMets,isNames);
    
    %If no soulution could be found, then finish
    if isempty(solution)
        break;
    end
    
    %Find all reactions that are unbalanced and still carry flux
    I=find(abs(solution)>10^-8 & balanceStructure.balanceStatus>=0 & ~all(balanceStructure.leftComp(:,bal)==balanceStructure.rightComp(:,bal),2));
    
    %If there are unbalanced rxns then delete one of them and iterate
    if any(I)
        rxnToRemove=I(randsample(numel(I),1));
    else
        %If there are no unbalanced rxns in the solution
        if rxnRules==1
            fprintf(['WARNING: No unbalanced reactions were found in the solution, but the model can still make "' model.metNames{metabolite} '". Aborting search. Consider setting rxnRules to 2 or 3 for a more exhaustive search\n']);
            break;
        else
            %Find reactions which are not checked for mass balancing, but
            %that still carry flux
            I=find(abs(solution)>10^-8 & balanceStructure.balanceStatus<0);

            %If there are any such reactions, remove one of them and
            %iterate
            if any(I)
                rxnToRemove=I(randsample(numel(I),1));
            else
                if rxnRules==2
                    %This happens when all reactions used are balanced
                    %according to the metabolite formulas. This cannot be, and
                    %indicates that one or more of the formulas are wrong.
                    %Print a warning and delete any reaction with flux
                    fprintf(['WARNING: No unbalanced or unparsable reactions were found in the solution, but the model can still make "' model.metNames{metabolite} '". Aborting search. Consider setting rxnRules to 3 for a more exhaustive search\n']);
                    break;
                else
                    fprintf(['WARNING: No unbalanced or unparsable reactions were found in the solution, but the model can still make "' model.metNames{metabolite} '". This indicates some error in the metabolite formulas. Removing a random reaction in the solution\n']);
                    I=find(abs(solution)>10^-8);
                    rxnToRemove=I(randsample(numel(I),1));
                end
            end
        end
    end
    removedRxns=[removedRxns;model.rxns(rxnToRemove)];
    model=removeRxns(model,rxnToRemove);
    balanceStructure.balanceStatus(rxnToRemove)=[];
    balanceStructure.leftComp(rxnToRemove,:)=[];
    balanceStructure.rightComp(rxnToRemove,:)=[];
end

newModel=model;
end